import 'package:flutter/material.dart';

class ParalaxListView extends StatefulWidget {
  final double flexibleHeight;
  final double floatingHeight;
  final Widget background;
  final Widget foreground;
  final List<Widget> chidrens;
  final Widget floatingWidget;
  final EdgeInsetsGeometry padding;
  final ScrollPhysics physics;

  ParalaxListView({
    this.flexibleHeight,
    this.floatingHeight,
    this.background,
    this.foreground,
    this.floatingWidget,
    this.chidrens,
    this.padding,
    this.physics,
  });

  @override
  _ParalaxListViewState createState() => _ParalaxListViewState();
}

class _ParalaxListViewState extends State<ParalaxListView> {
  ScrollController _scrollController = ScrollController();
  double _backgroundTopPosition = 0;

  @override
  void initState() {
    super.initState();
    _scrollController.addListener(_onScroll);
  }

  _onScroll() {
    double floatingHeight = widget.floatingHeight ?? 80;
    double release = (floatingHeight * 2) + 32;

    if (_scrollController.offset < floatingHeight + 32) {
      double minus =
          (_scrollController.offset * (_scrollController.offset / release));
      setState(() {
        _backgroundTopPosition = 0 - minus;
      });
    } else {
      setState(() {
        _backgroundTopPosition =
            0 - _scrollController.offset + (floatingHeight / 2) + 8;
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    double height = widget.flexibleHeight ?? 250;
    double floatingHeight = widget.floatingHeight ?? 80;
    List<Widget> childrens = [
      Container(
        width: MediaQuery.of(context).size.width,
        height: height - (floatingHeight / 2),
        child: SafeArea(
          child: widget.foreground,
        ),
      ),
      Container(
        width: MediaQuery.of(context).size.width,
        height: floatingHeight,
        child: widget.floatingWidget,
      ),
    ];

    if (widget.chidrens != null && widget.chidrens.isNotEmpty) {
      childrens.addAll(widget.chidrens);
    }

    return Stack(
      children: <Widget>[
        Positioned(
          top: _backgroundTopPosition,
          left: 0,
          right: 0,
          child: Container(
            height: height,
            child: widget.background,
          ),
        ),
        ListView(
          padding: widget.padding ?? const EdgeInsets.all(0),
          controller: _scrollController,
          physics: widget.physics ?? const BouncingScrollPhysics(),
          children: childrens,
        ),
      ],
    );
  }
}
